###############################################################################
## Company      : 4JS
## Program      : fgldde.4gl
## Date         : 09/01/1997
## Version      : 1.00
##
## Title        : Interface between "DDE for Windows" and "4GL-program"
##                WARNING! When do you use WINWORD or EXCEL, instructions
##                are langage  sensitive (German, French, English...)
## UPDATE 26/02/98:
##                err_flag and err_mess not more globals
###############################################################################

  
  DEFINE err_flag SMALLINT,
         err_mess CHAR(1024)


###############################################################################
##                                  FUNCTION
###############################################################################

################################################################################
## FUNCTION WinExec : Start a program in Windows side without waiting
## Parameters :
##  progname:  Program name with or without absolute path
##             Be careful with banalisation.
##             example: "C:\\\\WINWORD\\\\WINWORD.EXE"
## Returnings :
##  TRUE if OK else FALSE then see err_mess
################################################################################

FUNCTION WinExec(progname)

  DEFINE progname CHAR(256),
         ret      SMALLINT

  LET ret = WindowsExec(progname,0)

  RETURN ret

END FUNCTION

################################################################################
## FUNCTION WinExecWait : Start a program in Windows side with waiting end 
##                        of Windows program
## Parameters :
##  progname:  Program name with or without absolute path
##             Be careful with banalisation.
##             example: "C:\\\\WINWORD\\\\WINWORD.EXE"
## Returnings :
##  TRUE if OK else FALSE then see err_mess
################################################################################

FUNCTION WinExecWait(progname)

  DEFINE progname CHAR(256),
         ret      SMALLINT

  LET ret = WindowsExec(progname,1)

  RETURN ret

END FUNCTION

################################################################################
## FUNCTION WindowsExec : Start a program in Windows side
## Parameters :
##  progname:  Program name with or without absolute path
##             Be careful with banalisation.
##             example: "C:\\\\WINWORD\\\\WINWORD.EXE"
##  wait_flag : If TRUE when wait that the Windows program is closed to 
##              continue the 4gl program
## Returnings :
##  TRUE if OK else FALSE then see err_mess
################################################################################

FUNCTION WindowsExec(progname,wait_flag)

  DEFINE progname  CHAR(256),
         cmd       CHAR(512),
         ret       CHAR(1024),
         wait_flag SMALLINT

  IF wait_flag THEN
    LET cmd="winexec -wait \"",progname CLIPPED,"\""
  ELSE
    LET cmd="winexec \"",progname CLIPPED,"\""
  END IF

  LET cmd = "set reterror [ catch { ", cmd clipped, " } msgerror ]"

  CALL fgl_uisendcommand(cmd)

  let ret = fgl_uiretrieve("reterror")
  IF ret = 1 THEN
    LET err_flag = TRUE
    CALL DDESeterror()
    RETURN FALSE
  END IF

  RETURN TRUE
    
END FUNCTION #WinExec

################################################################################
## FUNCTION DDEConnect : Make the DDE connexion for Word
## Parameters :
##  progname : Program where do you want DDE connexion
##             example: WINWORD, EXCEL
##  docname  : Name of the current document to proceed or "System",
##             without any path
##             To manage all the program, docname="System"
##             In German, first document is Dokument1
## Both parameters determines a unique identificator
## Returnings :
##  TRUE if OK else FALSE then see errmess
################################################################################

FUNCTION DDEConnect(progname,docname)

  DEFINE progname  CHAR(128),
         docname   CHAR(128)
  DEFINE cmd       CHAR(1024)
define ret char(80)


###########################
# Make the dde connexion #
###########################
  LET cmd="ddeclient dde",progname CLIPPED,"_",docname CLIPPED,
           " ",progname CLIPPED," ",docname CLIPPED

  LET cmd = "set reterror [ catch { ", cmd clipped, " } msgerror ]"

  CALL fgl_uisendcommand(cmd)


  LET ret = fgl_uiretrieve("reterror")
  IF ret = 1 THEN
    LET err_flag = TRUE
    CALL DDESeterror()
    RETURN FALSE
  END IF
  RETURN TRUE

END FUNCTION

################################################################################
## FUNCTION DDEExecute : Execute a command into the Windows program
## Parameters :
##  progname  : Program name
##  docname   : document name or "System"
##  execcmd   : Command to execute
##              Be careful with banalisation.
## Returnings :
##  TRUE if OK else FALSE then see errmess
################################################################################

FUNCTION DDEExecute(progname,docname,execcmd)

  DEFINE progname  CHAR(128),
         docname   CHAR(128),
         execcmd   CHAR(2048)
  DEFINE cmd       CHAR(4096)
define ret char(80)


  LET cmd="dde",progname CLIPPED,"_",docname CLIPPED,
#          " execute \"",execcmd CLIPPED,"\""
          " execute \"\\[",execcmd CLIPPED,"\\]\""

  LET cmd = "set reterror [ catch { ", cmd clipped, " } msgerror ]"
  CALL fgl_uisendcommand(cmd)

  let ret = fgl_uiretrieve("reterror")
  IF ret = 1 THEN
    LET err_flag = TRUE
    CALL DDESeterror()
    RETURN FALSE
  END IF
  RETURN TRUE


END FUNCTION

################################################################################
## FUNCTION DDEPoke : Put values into Windows program
## Parameters :
##  progname  : Program name
##  docname   : document name or "System"
##  cells     : Cells to fill
##              by ex: R1C1:R2C12 fills cell A1,A2,...,A12,B1,B2,...,B12
##		format:
##                in German : Z#S#:Z#S# ( # represents numbers )
##                in English: R#C#:R#C#
##                in French : L#C#:L#C#
##  values    : values of cells separate with "\t"
##              by ex: "12\\t13\\t14"
##              Be careful with banalisation.
## Returnings :
##  TRUE if OK else FALSE then see errmess
################################################################################

FUNCTION DDEPoke(progname,docname,cells,values)

  DEFINE progname  CHAR(128),
         docname   CHAR(128),
         cells     CHAR(128),
         values    CHAR(8192)
  DEFINE cmd       CHAR(1024)
  DEFINE ret       CHAR(80)


  LET cmd="dde",progname CLIPPED,"_",docname CLIPPED,
          " poke ",cells CLIPPED," \"",values CLIPPED,"\""
  LET cmd = "set reterror [ catch { ", cmd clipped, " } msgerror ]"
  CALL fgl_uisendcommand(cmd)

  LET ret = fgl_uiretrieve("reterror")
  IF ret = 1 THEN
    LET err_flag = TRUE
    CALL DDESeterror()
    RETURN FALSE
  END IF
  RETURN TRUE

END FUNCTION

################################################################################
## FUNCTION DDEPeek : get data from Windows program
## Parameters :
##  progname  : Program name
##  docname   : document name or "System"
##  cells     : Cells to fill
##              by ex: R1C1:R2C12 fills cell A1,A2,...,A12,B1,B2,...,B12
##		format:
##                in German : Z#S#:Z#S# ( # represents numbers )
##                in English: R#C#:R#C#
##                in French : L#C#:L#C#
## Returnings :
##  values    : Return values from application
################################################################################

FUNCTION DDEPeek(progname,docname,cells)

  DEFINE progname  CHAR(128),
         docname   CHAR(128),
         cells     CHAR(128)
  DEFINE cmd       CHAR(1024)
  DEFINE ret       SMALLINT,
         value     CHAR(512)
define a char(1)


  LET cmd="dde",progname CLIPPED,"_",docname CLIPPED,
          " get ",cells CLIPPED
  LET cmd = "set reterror [ catch { ", cmd clipped, " } msgerror ]"
  CALL fgl_uisendcommand(cmd)


  let value = fgl_uiretrieve("reterror")
  IF value = 1 THEN
    LET err_flag = 1
    CALL DDESeterror()
    LET cmd = NULL
  ELSE
    LET cmd = fgl_uiretrieve("msgerror")
  END IF

  RETURN cmd

END FUNCTION

################################################################################
## FUNCTION DDEFinish : Terminate a DDE connexion
## Parameters :
##  progname  : Program name
##  docname   : document name or "System"
## Returnings :
##  TRUE if OK else FALSE then see errmess
################################################################################

FUNCTION DDEFinish(progname,docname)

  DEFINE progname  CHAR(128),
         docname   CHAR(128)
  DEFINE cmd       CHAR(1024)
define ret char(80)


  LET cmd="dde",progname CLIPPED,"_",docname CLIPPED,
          " terminate "
  LET cmd = "set reterror [ catch { ", cmd clipped, " } msgerror ]"
  CALL fgl_uisendcommand(cmd)

  LET ret = fgl_uiretrieve("reterror")
  IF ret = 1 THEN
    LET err_flag = TRUE
    CALL DDESeterror()
    RETURN FALSE
  END IF

  RETURN TRUE

END FUNCTION

################################################################################
## FUNCTION DDEFinishAll : Terminate all DDE connexion
## Parameters :
##  none
## Returnings :
##  none
################################################################################

FUNCTION DDEFinishAll()

  CALL fgl_uisendcommand("ddeterminateall")

END FUNCTION

################################################################################
## FUNCTION DDESeterror: put the error message in a global variable
## Parameters :
##  none
## Returnings :
##  none
################################################################################

FUNCTION DDESeterror()

  LET err_mess = fgl_uiretrieve("msgerror")

END FUNCTION


################################################################################
## FUNCTION DDEGeterror: return the error message 
## Parameters :
##  none
## Returnings :
##  none
################################################################################

FUNCTION DDEGeterror()

  DEFINE cmd CHAR(1024)

  IF err_flag THEN
    LET err_flag = FALSE
    LET cmd = err_mess CLIPPED
    LET err_mess = NULL
  ELSE
    LET cmd = NULL
  END IF
  
  RETURN cmd

END FUNCTION
